/*
 * wap.c - Generic functions for wap library
 */

#include "gwlib/gwlib.h"
#include "wap.h"
#include "wtp.h"

#define CONNECTIONLESS_PORT 9200
#define CONNECTIONLESS_PUSH_CLIPORT 2948

void wap_dispatch_datagram(WAPEvent *dgram) {
    gw_assert(dgram != NULL);

    if (dgram->type != T_DUnitdata_Ind) {
        warning(0, "wap_dispatch_datagram got event of unexpected type.");
        wap_event_dump(dgram);
        wap_event_destroy(dgram);
        return;
    }

    /* MJ - ASL add WAP client functionality */
    if (dgram->u.T_DUnitdata_Ind.addr_tuple->local->port == CONNECTIONLESS_PORT) {
        wsp_unit_dispatch_event(dgram);
    }
    else if (dgram->u.T_DUnitdata_Ind.addr_tuple->local->port == CONNECTIONLESS_PUSH_CLIPORT) {
        wsp_unit_client_dispatch_event(dgram);
    } 
    else {
        List *events;

        events = wtp_unpack_wdp_datagram(dgram);
        wap_event_destroy(dgram);
        while (list_len(events) > 0) {
            WAPEvent *event;

            event = list_extract_first(events);
            wtp_initiator_dispatch_event(event);
        }
        list_destroy(events, NULL);
    }
}
